package piy;

import piy.support.*;
import java.util.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.File;

/**
* This frame contains ClassSupport panels which are used to edit the properties of
* particular components.
* @author David Vivash
* @version 1.0, 08/02/01
*/
public class PropertyEditorFrame extends JFrame implements Observer, ActionListener {
	
	private Object selected				= null; //the currently selected component
	private PropertyManager manager		= null; //the property manager
	private ListenerManager listeners	= null; //the listener manager

	private JComboBox names = null;
	private static final String NAMES			= "Names"; //action command of the names comboBox
	private static final String DELETE_SELECTED	= "Delete"; //action command of the delete button


	private Container propertiesPane	= null; //the content pane of the properties
	private Container listenersPane		= null; //the content pane of the listeners

	private boolean editting = false; //set to true when the entries in the names combo box are being editted

	/**
	* Construct a new property editor which uses the specified property manager to
	* check components for properties.
	* @param manager the PropertyManager that can be used to introspect a component's 
	* properties
	* @param listeners the ListenerManager which habdles which events can be listened 
	* for on particular objects
	*/
	public PropertyEditorFrame(PropertyManager manager, ListenerManager listeners) {
		super("Properties");
		this.manager	= manager;
		this.listeners	= listeners;

		propertiesPane	= new JPanel();
		listenersPane	= new JPanel();

		JScrollPane pScrollPane = new JScrollPane(propertiesPane);
		JScrollPane lScrollPane = new JScrollPane(listenersPane);

		pScrollPane.setBorder(new EmptyBorder(0,0,0,0));
		lScrollPane.setBorder(new EmptyBorder(0,0,0,0));


		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Properties", pScrollPane);
		tabbedPane.addTab("Events", lScrollPane);

		//names combo box with a delete button to the left
		names = new JComboBox();
		names.addActionListener(this);
		
		JPanel comboPanel = new JPanel();
		comboPanel.setLayout(new BorderLayout());
		comboPanel.add(names, BorderLayout.CENTER);
		
		String sep = File.separator;
		String images = "piy" + sep + "images" + sep;

		FloatingButton delSelected = new FloatingButton(new ImageIcon(images + "delete.gif").getImage(), DELETE_SELECTED, false);
		delSelected.addActionListener(this);
		delSelected.setToolTipText("Delete Selected");
		delSelected.setPreferredSize(new Dimension(24,24));

		comboPanel.add(delSelected, BorderLayout.WEST);
		
		Container container = getContentPane();
		container.setLayout(new BorderLayout());

		container.add(comboPanel, BorderLayout.NORTH);
		container.add(tabbedPane, BorderLayout.CENTER);

		//update the view by showing the properties of the first user edit window
		PIYNamer windows = ProjectHandler.getInstance().getProject().getGuiNamer();
		updateView(selected = windows.getValue(windows.getNames()[0]));

		PIY.getInstance().addObserver(this);
	}

	/**
	* Updates the properties on display in the window.
	* @param selected the currently selected object whose properties will be displayed for editting
	*/
	private void updateView(Object selected) {

		ClassSupport[] support = null;

		propertiesPane.removeAll(); //get rid of old property editors
		listenersPane.removeAll(); //get rid of old listeners

		editting = true;
		//Update the choice list names (since a name could have been changed, or a 
		//component deleted
		names.removeAllItems();
		PIYNamer namer = ProjectHandler.getInstance().getProject().getGuiNamer();
		java.util.List windowNames = namer.getNamesList(UserWindow.class);
		java.util.List componentNames = namer.getNamesList(UserComponent.class);
		//merge the two lists and convert to an array of strings
		windowNames.addAll(componentNames);
		String[] allNames = namer.getNames(windowNames); //misleading method name -> relocate method to a utils class or something

		for (int i=0; i<allNames.length; i++)
			names.addItem(allNames[i]);

		names.insertItemAt("", 0); //insert a blank selection at the top of the list		

		if (selected != null) names.setSelectedItem(getName());
		else names.setSelectedIndex(0);

		names.setActionCommand(NAMES);


		//Sort out the properties of the selected component/container
		if (selected != null) {
	
			//use the property manager to get the edittable properties
			if (selected instanceof UserComponent) 
				support = manager.getClassSupport(selected, UserComponent.class, true, this);
			else
				support = manager.getClassSupport(selected, UserWindow.class, true, this);
	
			GridBagLayout gridbag = new GridBagLayout();
			GridBagConstraints c = new GridBagConstraints();
	
			propertiesPane.setLayout(gridbag);
			c.fill = GridBagConstraints.HORIZONTAL;
			c.ipadx = 4; c.ipady = 8;
			c.insets = new Insets(4,4,4,4);
	
			
			//Set up the "name" property first - force it to come through this class
			Property nameProperty = new Property(this, "Name", String.class);
			JLabel nameLabel = new JLabel("Name");
			c.weightx = 0;
			gridbag.setConstraints(nameLabel, c);
			propertiesPane.add(nameLabel);
			
			ClassSupport nameSupport = new StringSupport(nameProperty);
			c.gridx = 1; c.weightx = 1.0;
			gridbag.setConstraints(nameSupport, c);
			propertiesPane.add(nameSupport);
	
			//Set up the "Alignment" property (if the selected component is not a window) -
			//force it to come through this class
			if (!(selected instanceof UserWindow)) {
	
				Property alignProperty = new Property(this, "Alignment", Align.class);
				JLabel alignLabel = new JLabel("Alignment");
				c.weightx = 0; c.gridx = 0; c.gridy = 1;
				gridbag.setConstraints(alignLabel, c);
				propertiesPane.add(alignLabel);
	
				ClassSupport alignSupport = new AlignSupport(alignProperty);
				c.gridx = 1; c.weightx = 1.0;
				gridbag.setConstraints(alignSupport, c);
				propertiesPane.add(alignSupport);
			}
	
	
			//Add the rest of the properties
			for (int i=0; i<support.length; i++) {
	
				c.gridy = i+2; c.gridx = 0; c.weightx = 0;
				JLabel label = new JLabel(support[i].getProperty().getName());
				gridbag.setConstraints(label, c);
				propertiesPane.add(label);
				
				c.gridx = 1; c.weightx = 1.0;
				gridbag.setConstraints(support[i], c);
				propertiesPane.add(support[i]);
			}
			
			//add a blank panel at the bottom to make sure the (real) properties are 
			//aligned to the top of the frame.
			c.gridy = support.length + 2; c.weighty = 1.0;
			JPanel blank = new JPanel();
			gridbag.setConstraints(blank, c);
			propertiesPane.add(blank);
	
			//------- Now set up the listeners ---------
			PiyEventListener[] lSupport = listeners.getListeners(selected);
	
			listenersPane.setLayout(gridbag);
			c.fill = GridBagConstraints.HORIZONTAL;
			c.ipadx = 0; c.ipady = 0; c.weighty = 0;
			c.insets = new Insets(0,0,0,0);
	
			String[] actionLists = ProjectHandler.getInstance().getProject().getActionListNamer().getNames();
	
			HashMap eventListeners = ProjectHandler.getInstance().getProject().getEventListeners(selected);
	
			//Add the listeners
			int pos = 0;
//			editting = true;
			for (int i=0; i<lSupport.length; i++) {
				String[] eventNames = lSupport[i].getEvents();
	
				HashMap namesToActionLists = (HashMap)eventListeners.get(lSupport[i].getClass());
	
	
				for (int j=0; j<eventNames.length; j++) {
		
					c.gridy = pos; c.gridx = 0; c.weightx = 0;
					JLabel label = new JLabel(eventNames[j]);
					gridbag.setConstraints(label, c);
					listenersPane.add(label);
					
					c.gridx = 1; c.weightx = 1.0;
					JComboBox temp = new JComboBox(actionLists);
					temp.insertItemAt("", 0);
	
					//Get the index
					if (namesToActionLists != null) {
						ActionList mappedTo = (ActionList)namesToActionLists.get(eventNames[j]);
						String name = ProjectHandler.getInstance().getProject().getActionListNamer().getName(mappedTo);
						temp.setSelectedItem(name);
					} else temp.setSelectedIndex(0);
					
					temp.setActionCommand(eventNames[j]);
					temp.addActionListener(this);
					temp.setPreferredSize(new Dimension(100, 24));
					gridbag.setConstraints(temp, c);
					listenersPane.add(temp);
	
					pos++;
				}
			}

			//add a blank panel at the bottom to make sure the (real) properties are 
			//aligned to the top of the frame.
			c.gridx = 0; c.gridy = pos; c.weighty = 1.0;
			JPanel blank2 = new JPanel();
			gridbag.setConstraints(blank2, c);
			listenersPane.add(blank2);
		}

		editting = false;	
		
		//make sure all components are laid out properly
		propertiesPane.validate(); 
		propertiesPane.repaint();
		listenersPane.validate(); 
		listenersPane.repaint();
	}

	/**
	* Set the name of the currently selected component.  This uses the current project's
	* Namer object to ensure naming clashes do not occur.
	* @param name the new name of the object
	*/
	public void setName(String name) {
		editting = true;
		PIYNamer namer = ProjectHandler.getInstance().getGuiNamer();
		String oldName = namer.getName(selected);
		namer.renameValue(oldName, name);
		String newName = namer.getName(selected);
		names.removeItem(oldName);
		names.addItem(newName);
		names.setSelectedItem(newName);
		editting = false;
	}

	/**
	* Get's the name of the currently selected component.
	* @return the component's name
	*/
	public String getName() {
		return ProjectHandler.getInstance().getProject().getName(selected);
	}

	// ------------------------------------------------------------------
	// We redirect the size and position properties through here to ensure the bounding 
	// box gets updated.  We can also ensure that the layout manager gets invoked
	// to keep the component in the correct place after is is moved or resized
	public Dimension getSizes() { return ((Component)selected).getSize(); }
	public Point getPosition() { return ((Component)selected).getLocation(); }
	public void setPosition(Point pos) {
		if (selected != null) {
			((Component)selected).setLocation(pos);
			UserWindow win = UserWindowListener.getInstance().getCurrentWindow();
			if (win != null) win.validate();
			UserWindowListener.getInstance().validateBoundingBox();
		}
	}
	public void setSizes(Dimension size) { 
		if (selected != null) {
			((Component)selected).setSize(size);
			UserWindow win = UserWindowListener.getInstance().getCurrentWindow();
			if (win != null) win.validate();
			UserWindowListener.getInstance().validateBoundingBox();
		}
	}
	//------------------------------------------------

	/**
	* Set the alignment of the currently selected component.
	* @param align the new alignment of the object
	*/
	public void setAlignment(Align align) {
		AlignmentLayout al = (AlignmentLayout)((Component)selected).getParent().getLayout();

		al.setAlignment((Component)selected, align.getAlign());

		al.layoutContainer(((Component)selected).getParent());

		UserWindowListener.getInstance().validateBoundingBox();
	}

	/**
	* Get's the alignment of the currently selected component.
	* @return the component's alignment
	*/
	public Align getAlignment() {
		AlignmentLayout al = (AlignmentLayout)(((Component)selected).getParent()).getLayout();
		return new Align(al.getAlignment((Component)selected));
	}

	//----- Observer Method -------
	public void update(Observable o, Object arg) {
		
		ActionEvent e = (ActionEvent)arg;

		if (e.getActionCommand().equals(PIY.COMPONENT_SELECTED) || 
			e.getActionCommand().equals(PIY.COMPONENT_CHANGED) ||
			e.getActionCommand().equals(PIY.COMPONENT_DESELECTED)) {

			selected = UserWindowListener.getInstance().getSelectedComponent();

			if (selected == null) 
				selected = UserWindowListener.getInstance().getCurrentWindow();

			updateView(selected);
		}
	}

	//------- ActionListener Method ----------
	public void actionPerformed(ActionEvent e) {
		if (!editting) {

			if (e.getActionCommand() == NAMES) { //A names combo box item was selected
				//"select" the relevant component on the relevant window
				String chosen = (String)names.getSelectedItem();
				
				if (chosen.equals("")) {
					UserWindowListener.getInstance().setCurrentWindow(null);				
					UserWindowListener.getInstance().selectComponent(null);								
				} else {
					Object toSelect = ProjectHandler.getInstance().getGuiNamer().getValue(chosen);
					Component win = (Component)toSelect;
					while (!(win instanceof UserWindow)) win = win.getParent();
	
					((JFrame)win).setVisible(true);
					((JFrame)win).requestFocus();
	
					if (win == toSelect) {
						toSelect = null; //toSelect is a window		
						UserWindowListener.getInstance().setCurrentWindow((JFrame)win);				
					}
					UserWindowListener.getInstance().selectComponent((JComponent)toSelect);
				}

			} else if (e.getActionCommand() == DELETE_SELECTED) {

				String chosen = (String)names.getSelectedItem();

				if (chosen.equals("")) {
				
				} else {				
					PIYNamer namer = ProjectHandler.getInstance().getGuiNamer();
					Component toDelete = (Component)namer.getValue(chosen);
					boolean clearView = true;
	
					if (toDelete instanceof UserWindow) {
						killChildren(((JFrame)toDelete).getContentPane()); //remove references to all things stored in the user window
						((JFrame)toDelete).getContentPane().removeAll(); //physically remove them
						
						boolean dispose = ProjectHandler.getInstance().getProject().removeWindow(chosen);//remove user window from project
						if (dispose) { //the window should be deleted, since it has been removed from the project
							((JFrame)toDelete).dispose();
							UserWindowListener.getInstance().setCurrentWindow(null);	
						}
						
						clearView = dispose;

					} else if (toDelete instanceof UserContainer) {
						Container parent = (toDelete).getParent();
						parent.remove(toDelete); //remove this component from its parent
						killChildren((Container)toDelete); //remove container and children from project
					} else {
						Container parent = (toDelete).getParent();
						parent.remove(toDelete);
						namer.removeValue(chosen); //remove component from project
						parent.repaint();
					}
	
					UserWindowListener.getInstance().selectComponent(null);

					if (clearView) updateView(null);

					JFrame curWindow = UserWindowListener.getInstance().getCurrentWindow();
					if (curWindow != null) curWindow.validate();
					
					UserWindowListener.getInstance().validateBoundingBox();
				
				}

			} else { //an actionlist has been selected

				String actionListName = (String)((JComboBox)e.getSource()).getSelectedItem();

				//get the event listener that listens on the selected event
				Class eventListener = getEventListener(e.getActionCommand());
	
				HashMap componentsToListeners = ProjectHandler.getInstance().getProject().getEventListeners(selected);
				HashMap namesToLists = (HashMap)componentsToListeners.get(eventListener);
				
				if (namesToLists == null) componentsToListeners.put(eventListener, namesToLists = new HashMap(2));

				ActionList actionList = (ActionList)ProjectHandler.getInstance().getProject().getActionListNamer().getValue(actionListName);
				namesToLists.put(e.getActionCommand(), actionList);
			}
		}
	}
		
	/**
	* Removes all of the children and the specified container from the piy project mappings.
	* @param container the container who should be removed
	*/	
	private void killChildren(Container holder) {
		PIYNamer namer = ProjectHandler.getInstance().getGuiNamer();
		namer.removeValue(namer.getName(holder));
		Component[] kids = holder.getComponents();
		
		for (int i=0; i<kids.length; i++)
			if (kids[i] instanceof Container) killChildren((Container)kids[i]);
			else namer.removeValue(namer.getName(kids[i]));
	}	
	
	/**
	* Gets the PIYEventListener class object for the specified event name.
	* @param eventName the name of the event, or null if the event is not 
	* found in any registered listeners
	*/
	private Class getEventListener(String eventName) {
		
		PiyEventListener[] eventListeners = listeners.getListeners(selected);
		
		for (int i=0; i<eventListeners.length; i++) {
			String[] events = eventListeners[i].getEvents();
			for (int j=0; j<events.length; j++)
				if (events[j] == eventName) //same reference
					return eventListeners[i].getClass();
		}
		
		return null;
	}
}