package piy;

import java.util.ArrayList;
import java.lang.reflect.*;

/**
* Uses ClassSupport to offer a property editting interface for users.  A "holder" class, ie. one
* with some properties to edit is fed in to this class, which determines which of the properties
* in "holder" can be editted by PIY, and can return an array of ClassSupport objects set
* up and ready to be used.
* @author David Vivash
* @version 1.0.2, 10/02/01
*/
public class PropertyManager
{
	/**
	* Each  of these classes is a ClassSupport class, which have not yet been instatiated.
	* To find out which classes they support, you must first instatiate them and then
	* call the "getSupportedType()" method.
	*/
	private Class[] support;

	/**
	* Start the property manager with the specified class support classes.
	* @param support the classes representing ClassSupport classes - each ClassSupport class
	* supports a particular "datatype"
	*/
	public PropertyManager(Class[] support) {
		this.support = support;
	}

	/**
	* Retrieves all of the class support objects (which are implicitly bound to the properties)
	* which offer editting support for all of the properties in the holder class.
	* @param holder the object for which to get property object support for
	* @param baseClass how far up the object heirarchy to go in retrieving properties
	* @param standard if the class is a standard GUI component, setting this to true ensures the position
	* and size are counted as properties
	* @param standardRoute if standard properties are being used, this should be set to null to imply
	* the properties get routed directly through the holder, or can be set to point to a different
	* object if the properties should be set indirectly.  Note:  the properties get rediected to
	* the names Szes and Position, to allow the route object to implement its own
	* Size and Position methods independently
	* @return an array containing all of the editors already set-up to edit the properties of holder
	*/
	public ClassSupport[] getClassSupport(Object holder, Class baseClass, boolean standard, Object standardRoute) {

		Property[] properties = getProperties(holder, baseClass, standard, standardRoute);

		ClassSupport[] toReturn = new ClassSupport[properties.length];

		//bind each class support object to the relevant property
		for (int i=0; i<properties.length; i++)
			toReturn[i] = getSupport(properties[i]);

		return toReturn;
	}

	/**
	* Retrieves all of the properties from the specified holder class, as well as from super classes
	* up to and including the baseClass.  All properties that have a suitable support class are
	* accepted.  Note that the properties returned will be bound to the supplied holder object 
	* properties
	* @param holder the object which holds the properties that are required
	* @param baseClass how far up the inheritance tree to go (supply baseClass=holder if only properties
	* from the holder class are required, or baseClass=Object if _all_ available properties are required)
	* @param support all of the type support classes available
	* @param standard if the class is a standard GUI component, setting this to true ensures the position
	* and size are counted as properties
	* @param standardRoute if standard properties are being used, this should be set to null to imply
	* the properties get routed directly through the holder, or can be set to point to a different
	* object if the properties should be set indirectly.  Note:  the properties get rediected to
	* the names Szes and Position, to allow the route object to implement its own
	* Size and Position methods independently
	* @return an array containing all property objects for the supplied holder object
	*/
	public Property[] getProperties(Object holder, Class baseClass, boolean standard, Object standardRoute) {
		Method[] allMethods = getMethods(holder.getClass(), baseClass);

		//retrieve all of the "get" and "set" methods
		Method[] setMethods = seiveMethods(allMethods, "set");
		Method[] getMethods = seiveMethods(allMethods, "get");

		ArrayList properties = new ArrayList(4);

		//------- All (standard) classes have these properties --------
		if (standard) {
			if (standardRoute == null) { //properties get bound directly to their holder
				properties.add(new Property(holder, "Location", java.awt.Point.class));
				properties.add(new Property(holder, "Sze", java.awt.Dimension.class));
			} else { //properties get redirected
				properties.add(new Property(standardRoute, "Position", java.awt.Point.class));
				properties.add(new Property(standardRoute, "Sizes", java.awt.Dimension.class));
			}
		}

		Method getSupportedClass = null;

		Class type = null;

		//go through all of the set methods, find the matching get method and then set up the property
		for (int i=0; i<setMethods.length; i++) {
			if (setMethods[i].getParameterTypes().length == 1) { //only look at the method if it has got one parameter

				type = setMethods[i].getParameterTypes()[0]; //first (only only) parameter type
				
				boolean foundType = false;

				for (int j=0; j<support.length && !foundType; j++)
					try{

						getSupportedClass = support[j].getMethod("getSupportedClass", null);

						if (type == getSupportedClass.invoke(null, new Object[0])) { //the type is supported
	
							foundType = true;
							String name = setMethods[i].getName().substring(3);
	
							//find the matching get method
							boolean foundMethod = false;
	
							for (int k=0; k<getMethods.length && !foundMethod; k++) {
	
								if (getMethods[k].getName().endsWith(name) &&
									getMethods[k].getReturnType() == type) {
	
									foundMethod = true;
									properties.add(new Property(holder, name, type));
								}
							}
						}
					} catch(Exception e) {
						//The methods must exist, so ignore these exceptions
						System.err.println("Exception trying to invoke method");
					}
			}
		}

		//copy into an array
		Property[] toReturn = new Property[properties.size()];
		for (int i=0; i<toReturn.length; i++)
			toReturn[i] = (Property)properties.get(i);

		return toReturn;
	}

	/**
	* Retrieves all of the methods declared in holder, as well as all of its subclasses up to
	* and including baseClass.
	* @param holder the class containing the methods
	* @param baseClass the highest superclass of holder that methods should be retrieved from
	* @return an array of methods that can be called on holder, but doesn't include mehods
	* declared in superclasses of baseClass
	*/
	private Method[] getMethods(Class holder, Class baseClass) {

		Class superClass = holder;
		ArrayList methods = new ArrayList(4);
		boolean add = true;

		while (baseClass.isAssignableFrom(superClass)) {

			Method[] methodsDeclared = superClass.getDeclaredMethods();

			for (int i=0; i<methodsDeclared.length; i++)
			{
				add = true;

				for (int j=0; j<methods.size(); j++)
				{

					Method check = (Method)methods.get(j);
			
					//if a method does have the same name, check it has a different set of parameters
					if ( check.getName().equals(methodsDeclared[i].getName()) ) {
						Class[] declaredTypes	= methodsDeclared[i].getParameterTypes();
						Class[] checkTypes		= check.getParameterTypes();

						if (declaredTypes.length == checkTypes.length) { //same number of parameters
							int parametersIdentical = 0; 
							for (int k=0; k<checkTypes.length; k++)
								if (declaredTypes[k].getName().equals(checkTypes[k].getName()))
									parametersIdentical++;

							if (parametersIdentical == declaredTypes.length)
								add = false;
			
						}
					}
				}

				if (add) methods.add(methodsDeclared[i]);
			}

			superClass = superClass.getSuperclass();		
		}

		Method[] toReturn = new Method[methods.size()];
		for (int i=0; i<methods.size(); i++)
			toReturn[i] = (Method)methods.get(i);

		return toReturn;
	}
	
	
	/**
	* Get the class support object for a single property.
	* @param property the property for which a class support object should be created
	* @return a class support supporting the supplied property, or null is no support was found
	*/
	public ClassSupport getSupport(Property property) {
		ClassSupport toReturn = null;
		Method getSupportedClass = null;

		for (int j=0; j<support.length; j++)
			try{
				getSupportedClass = support[j].getMethod("getSupportedClass", null);

				if (property.getType() == (Class)getSupportedClass.invoke(null, null)) {
					Constructor constructor = support[j].getConstructor(new Class[] { Property.class });
					toReturn = (ClassSupport)(constructor.newInstance(new Object[] { property }));
				}

			} catch (Exception e) {
				//since the relevant constructor DOES exist, and we're allowed to instantiate it,
				//we can ignore this exception, albeit with a warning 
				//(there must be an error in the constructor of the support class somewhere):
				System.err.println("Error instantiating a ClassSupport class in the PropertyManager"); 
			}
	
		return toReturn;
	}

	/**
	* Removes all methods from the input set that do not start with the start string
	* @param methods an array of method objects to be seived
	* @param start the starting string all returned method names should start with
	* @return an array of methods containing only method names starting with the start string
	*/
	private Method[] seiveMethods(Method[] methods, String start) {
		ArrayList seived = new ArrayList(methods.length);

		for (int i=0; i<methods.length; i++)
			if (methods[i].getName().startsWith(start))
				seived.add(methods[i]);		
		
		Method[] toReturn = new Method[seived.size()];
		for (int i=0; i<toReturn.length; i++)
			toReturn[i] = (Method)seived.get(i);

		return toReturn;
	}
}